"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const abi_1 = require("@ethersproject/abi");
const crypto_1 = require("crypto");
const fs = tslib_1.__importStar(require("fs"));
const web3_eth_accounts_1 = tslib_1.__importDefault(require("web3-eth-accounts"));
const web3_providers_http_1 = tslib_1.__importDefault(require("web3-providers-http"));
const knownAbis_1 = require("./knownAbis");
exports.add0x = (hex) => {
    return hex.indexOf('0x') === 0 ? hex : `0x${hex}`;
};
exports.loadABI = (abiPath) => {
    // Try to get the abi from the default list of supported abi's
    let abiStr = knownAbis_1.getStringAbiByName(abiPath);
    // If not found, check if the given string is a path to a file
    if (!abiStr) {
        if (fs.existsSync(abiPath)) {
            abiStr = fs.readFileSync(abiPath).toString();
            console.trace(abiStr);
        }
        else {
            // if it's not a file, check if it's a human readable ABI
            const iface = new abi_1.Interface([abiPath]);
            // fix for null components
            const fragment = JSON.parse(JSON.stringify(iface.fragments[0]));
            const inputs = fragment.inputs.map((x) => {
                if (x.components === null) {
                    delete x.components;
                }
                return x;
            });
            const outputs = fragment.outputs.map((x) => {
                if (x.components === null) {
                    delete x.components;
                }
                return x;
            });
            const fixedFragment = Object.assign(Object.assign({}, fragment), { inputs,
                outputs });
            abiStr = JSON.stringify([fixedFragment]);
        }
    }
    let abi = null;
    try {
        abi = JSON.parse(abiStr);
        // Allow using truffle artifacts files too.
        // If abi variable it's an object and it has an abi property, interpret it as a truffle artifact
        if (abi.abi) {
            abi = abi.abi;
        }
    }
    catch (e) {
        // tslint:disable-next-line:no-console
        console.error('Error parsing abi', e);
        process.exit(1);
    }
    return abi;
};
exports.extractMethodsAndEventsFromABI = (abi) => {
    return abi
        .filter((x) => x.type === 'function' || (x.type === 'event' && 'name' in x))
        .map(({ name, inputs, type }) => ({
        name,
        inputs,
        kind: type === 'function' ? 'function' : 'event',
    }));
};
/**
 * Evaluates a method call structure and returns an object with the information validated
 */
exports.evaluateMethodCallStructure = (methodCall) => {
    const isValidMethod = /^(\w+)\((.*)\)$/;
    const method = isValidMethod.exec(methodCall);
    return {
        methodCall,
        methodName: method ? method[1] : null,
        methodArgs: method ? method[2] : null,
        methodValid: !!method,
    };
};
exports.generateAccount = (prefix) => () => {
    let account = createAccount();
    while (account.address.slice(2, 2 + prefix.length) !== prefix) {
        account = createAccount();
    }
    return account;
};
exports.range = (amount) => new Array(amount).fill(true);
exports.evaluatePrefix = (prefix) => {
    const isValidPrefix = /(^$|^[a-fA-F0-9]+)$/;
    const match = isValidPrefix.exec(prefix);
    return match ? match[1] : null;
};
const createAccount = () => {
    const { wallet } = new web3_eth_accounts_1.default(new web3_providers_http_1.default(''));
    return wallet.create(1, crypto_1.randomBytes(32).toString('hex'))[0];
};
exports.isBN = (x) => x._hex !== undefined;
exports.isPrivateKey = (s) => {
    return /^(0x)?[0-9a-fA-F]{64}$/.test(s);
};
exports.isAddress = (s) => {
    return /^(0x)?[0-9a-fA-F]{40}$/.test(s);
};
// tslint:disable-next-line no-string-based-set-timeout
exports.sleep = (timeout) => new Promise(res => setTimeout(res, timeout));
